import { Command } from 'ckeditor5/src/core';

export default class SplitAndCreateCommand extends Command {

  /**
   * @inheritDoc
   */
  refresh() {
    this.isEnabled = !!this.getEditedComponentUuid() && !!this.getComponentSaveButton();
  }

  execute() {
    const { model } = this.editor;
    const splitMarker = Drupal.t('[Splitting in progress... ⌛]');
    const originalText = this.editor.getData();

    model.change(writer => {
      writer.insertText(splitMarker, model.document.selection.getFirstPosition());
    });

    const rootElement = (new DOMParser()).parseFromString(this.editor.getData(), 'text/html').body;
    const [elementBefore, elementAfter, markerFound] = SplitAndCreateCommand.splitNode(rootElement, splitMarker);
    if (!markerFound || !elementBefore || !elementAfter) {
      this.editor.setData(originalText);
      return;
    }

    // Store the elements we'll need later, after CKE widget has been closed.
    window._layoutParagraphsSplit = {
      uuidToSplit: this.getEditedComponentUuid(),
      end: elementAfter,
      nextStep: 'duplicate',
    };

    // Only keep the first part in the current paragraph.
    this.editor.setData(elementBefore.outerHTML);

    // Save this cut paragraph.
    this.getComponentSaveButton().click();

    // Next step is handled in the non-CKE Drupal JS library.
  }


  getComponentSaveButton() {
    return this.editor.sourceElement.closest('body').querySelector('.lpb-dialog.ui-widget button.lpb-btn--save');
  }

  getEditedComponentUuid() {
    return this.editor.sourceElement.closest('.layout-paragraphs-component-form')?.action.split('/').pop();
  }

  /**
   * Copied from paragraphs_features module split feature.
   */
  static splitNode(node, splitMarker) {
    const nestedSplitter = (n) => {
      if (n.nodeType === Node.TEXT_NODE) {
        // Split position within text node.
        const markerPos = n.data.indexOf(splitMarker);
        if (markerPos >= 0) {
          const textBeforeSplit = n.data.substring(0, markerPos);
          const textAfterSplit = n.data.substring(markerPos + splitMarker.length);

          return [
            textBeforeSplit ? document.createTextNode(textBeforeSplit) : null,
            textAfterSplit ? document.createTextNode(textAfterSplit) : null,
            true,
          ];
        }

        return [n, null, false];
      }

      const childNodesBefore = [];
      const childNodesAfter = [];
      let found = false;
      n.childNodes.forEach((childNode) => {
        // Split not yet reached.
        if (!found) {
          const [childNodeBefore, childNodeAfter, markerFound] = nestedSplitter(childNode);
          found = markerFound;

          if (childNodeBefore) {
            childNodesBefore.push(childNodeBefore);
          }

          if (childNodeAfter) {
            childNodesAfter.push(childNodeAfter);
          }
        }
        else {
          childNodesAfter.push(childNode);
        }
      });

      // Node was not split.
      if (!found) {
        return [n, null, false];
      }

      const nodeBefore = n.cloneNode();
      const nodeAfter = n.cloneNode();

      childNodesBefore.forEach((childNode) => {
        nodeBefore.appendChild(childNode);
      });

      childNodesAfter.forEach((childNode) => {
        nodeAfter.appendChild(childNode);
      });

      return [
        nodeBefore.childNodes.length > 0 ? nodeBefore : null,
        nodeAfter.childNodes.length > 0 ? nodeAfter : null,
        found,
      ];
    };

    return nestedSplitter(node);
  }
}

